/*
 * $Id: libnetwrite.h,v 1.1 2006/03/15 01:55:26 gomor Exp $
 *
 * AUTHOR    
 * 
 * Patrice <GomoR> Auffret
 *          
 * COPYRIGHT AND LICENSE
 *    
 * Copyright (c) 2006, Patrice <GomoR> Auffret
 *         
 * You may distribute this module under the terms of the Artistic license.
 * See Copying file in the source distribution archive.
 */

int
netwrite_open(char *interface);
